/*****************************************************************************
*
* File              : TWI_led_slave.c
*							 TAB = 3
* AppNote           : AVR311 / 315
*
* nderungen        : Enthlt die Fuktionalit eines Slaves.
*							 Luft als (vereinfachter) Slave unabhngig von main():
*							 Es sind nur die Empfangsroutinen aktiv.
*							 Daten, die empfangen werden, werden direkt in einen Puffer geschrieben.
*
****************************************************************************/

#include <avr/interrupt.h>              
#include <avr/io.h>
#include <avr/eeprom.h>
#include <inttypes.h>
#include "TWI_led_slave.h"
#include "global.h"

/*
extern uint8_t led_value[LEDs];
extern uint8_t iX;
extern uint8_t iP;
extern uint8_t ausgangswert[8];

extern uint8_t led_bitcode[LEDs];
*/
// Verwendet das globale Array twi_buffer[] mit den Index twi_idx
// rx_flag wird auf TRUE gesetzt, wenn der Slave schreibend addressiert wird
volatile uint8_t twi_idx;
volatile uint8_t rx_flag = FALSE;

// ----------------------------------------------------------------------------
// Initailisierung fr den Slave Modus
void TWI_init(void)
{
	TWAR = TWI_SLA_ADR;
   TWBR = TWI_TWBR;                             // Set bit rate register (Baudrate). Defined in header file.
// TWSR = TWI_TWPS;                             // Not used. Driver presumes prescaler to be 00.
//	TWDR = 0xFF;                                 // Default content = SDA released.
	TWCR = (1<<TWEN)|                            // TWI Interface enabled.
          (1<<TWIE)|(1<<TWINT)|                 // Enable TWI Interupt and clear the flag.
          (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|      // Prepare to ACK next time the Slave is addressed.
          (0<<TWWC); 

}

// -----------------------------------------------------------------------------
// gibt TRUE zurck, wenn fr jede LED ein Wert sowie ein zustzliches Steuerbyte empfangen wurde.
// Ist das erste Byte == 1 und sind 2 Byte bertragen (Variable pwm_wait), wird ebenfalls Erfolg gemeldet.
// Das erste Byte ist das Steuerbyte, das bei
// -> 0 die Helligkeitswerte der LEDs  
// -> 1 die Variable pwm_wait in Byte 2
// alle anderen Werte geben die Anzahl der bergnge zum neuen Helligkeitswert an.

uint8_t TWI_ready(void)
{
// zum Beginn einer bertragung wird rx_flag auf TRUE gesetzt
if (!(rx_flag)) return(FALSE);

/*
// wenn Steuerzeichen == 1 und ein zustzlicher Parameter
if ((twi_buffer[0] == 1) && (twi_idx > 2)) return(TRUE);

// wenn Steuerzeichen == 1 und ein zustzlicher Parameter
if ((twi_buffer[0] == 2) && (twi_idx > 1)) return(TRUE);
*/

// wenn Steuerzeichen > 0 und soviele Helligkeitswerte wie LEDs definiert
if (twi_idx > LEDs) return(TRUE);
// in allen anderen Fllen
return(FALSE);
}

// -----------------------------------------------------------------------------
// lscht die empfangenen Daten und setzt den TWI_Index auf 0,
// damit TWI_ready() keinen Erfolg hat
void TWI_clear_received(void)
{
uint8_t i;
twi_idx = 0;
for (i = 0; i < LEDs; i++) twi_buffer[i] = 0;
rx_flag = FALSE;
}

/****************************************************************************
This function is the Interrupt Service Routine (ISR), and called when the TWI interrupt is triggered;
that is whenever a TWI event has occurred. This function should not be called directly from the main
application.
****************************************************************************/

ISR(TWI_vect)
{
	switch (TWSR)
	{
// TWI Slave Transmitter status codes				// ###############################################

   case TWI_STX_ADR_ACK:            				// Own SLA+R has been received; ACK has been returned
//		twi_idx = 0;                    				// buffer pointer immer auf das 1. Element

// der Slave hat ein Byte gesendet, der Master mit ACK quittiert
   case TWI_STX_DATA_ACK:           				// Data byte in TWDR has been transmitted; ACK has been received
																// Es werden solange Daten gesendet, bis der Master mit NACK beendet
		TWDR = pwm_busy;									// gibt zurck, ob gerade ein bergang in Arbeit ist

      TWCR =(1<<TWEN)|                         	// TWI Interface enabled
          (1<<TWIE)|(1<<TWINT)|              	// Enable TWI Interupt and clear the flag to send byte
            (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|   	// Send ACK
            (0<<TWWC);                         	//  
      break;


// TWI Slave Receiver status codes					// ###############################################

// der Slave ist im read-Modus und hat gerade seine eigene Adresse empfangen
// eigene Slave-Adresse + write empfangen und mit ACK quittiert
   case TWI_SRX_ADR_ACK: 								// Own SLA+W has been received ACK has been returned
		twi_idx = 0; 
		rx_flag = TRUE;
		TWCR = (1<<TWEN)|                         // TWI Interface enabled
             (1<<TWIE)|(1<<TWINT)|              // Enable TWI Interupt and clear the flag to send byte
             (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|   // Send ACK after next reception
             (0<<TWWC);                         // 
		
		break;
			
// Slave im read Modus, Byte empfangen und mit ACK quittiert, nun das Byte ablegen
   case TWI_SRX_ADR_DATA_ACK:       				// Previously addressed with own SLA+W; data has been received; 
																// ACK has been returned
		twi_buffer[twi_idx++] = TWDR;					// ein Byte wurde empfangen und in den Puffer eingereiht.
		if (twi_idx > (TWI_BUFFER_SIZE -1)) twi_idx--; // auf berlauf testen

		TWCR = (1<<TWEN)|                         // TWI Interface enabled
             (1<<TWIE)|(1<<TWINT)|              // Enable TWI Interupt and clear the flag to send byte
             (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|   // Send ACK after next reception
             (0<<TWWC);                         //  
		break;

// case TWI_STX_DATA_NACK:          				// Data byte in TWDR has been transmitted; NACK has been received. 
																// I.e. this could be the end of the transmission.
//	case TWI_STX_ADR_ACK_M_ARB_LOST: 				// Arbitration lost in SLA+R/W as Master; 
																// own SLA+R has been received; ACK has been returned
// case TWI_STX_DATA_ACK_LAST_BYTE: 				// Last data byte in TWDR has been transmitted (TWEA = 0); 
																// ACK has been received
// case TWI_SRX_ADR_DATA_NACK:      				// Previously addressed with own SLA+W; data has been received; 
																// NOT ACK has been returned
// case TWI_SRX_GEN_DATA_NACK:      				// Previously addressed with general call; 
																// data has been received; NOT ACK has been returned
// case TWI_SRX_GEN_ACK:            				// General call address has been received; ACK has been returned

// case TWI_SRX_GEN_DATA_ACK:       				// Previously addressed with general call; 
																// data has been received; ACK has been returned
// case TWI_SRX_ADR_ACK_M_ARB_LOST: 				// Arbitration lost in SLA+R/W as Master; 
																// own SLA+W has been received; ACK has been returned 
// case TWI_SRX_GEN_ACK_M_ARB_LOST: 				// Arbitration lost in SLA+R/W as Master; 
																// General call address has been received; ACK has been returned
// case TWI_SRX_STOP_RESTART:       				// A STOP condition or repeated START condition has been received 
																// while still addressed as Slave 
//	case TWI_NO_STATE 									// No relevant state information available; TWINT = 0
// case TWI_BUS_ERROR:         						// Bus error due to an illegal START or STOP condition

// im Slave-Modus laufen alle anderen Flle ber default:
   default:     
     
		TWCR = (1<<TWEN)|                         // TWI Interface enabled
             (1<<TWIE)|(1<<TWINT)|              // Enable TWI Interupt and clear the flag to send byte
             (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|   // Send ACK after next reception
             (0<<TWWC);                      
		break; 
	}
}

// -- [ eof ] -----------------------------------------------------------------